#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "ctrie.h"

void print_val(char * key, void * data)
   {
   printf("%s\t\t%s\n", key, data);
   }

int main(int argc, char ** argv)
   {
   struct ctrie * ctrie = ctrie_new();

   char * words[] = { "hello", "interjection (used to express a greeting, answer a telephone, or attract attention.)",
		      "help", "to give or provide what is necessary to accomplish a task or satisfy a need.",
		      "helper", "a person or thing that helps  or gives assistance, support, etc",
		      "helping", "the act of a person or thing that helps.",
		      "an", "indefinite article. the form of a before an initial vowel sound.",
		      "a", "indefinite article.  Used before an initial consonant sound.",
		      "anne", "a female name",
		      "any", "one, a, an, or some; one or more without specification or identification",
		      "anyway", "in any case; anyhow; nonetheless; regardless",	
		      "anna", "a female name",
		      "anyhow", "in any  way whatever",
		      "anybody", "any person",
		      "Apple", "A computer manufacturer.",
		      "ABLE", "having necessary power, skill, resources, or qualifications",
		      /*
		      */
		      NULL, NULL
   };
   int i = 0;
   while (words[i])
      {
      printf("Adding %s\n", words[i]);
      ctrie_insert(ctrie, words[i], words[i+1]);
      i += 2;
      }

   ctrie_dump_raw(ctrie);
   ctrie_dump(ctrie);
   ctrie_walk_keys(ctrie, print_val);
   printf("ctrie is storing %zd words in %zd keys\n", ctrie_count(ctrie), ctrie_size(ctrie));

   exit(EXIT_SUCCESS);

   ////////////////////////////////////////////
   printf("===================================================\n");
   char word[100];;
   strcpy(word, "any");
   printf("searching for %s\n", word);

   void * data = ctrie_find(ctrie, word);
   if (NULL == data)
      printf("Failed to find 'any'\n");
   else
      printf("any: %s\n", (char *) data);


   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "an");
   printf("Looking for completions of '%s':\n", word);
   struct list * list = ctrie_get_subkeys(ctrie, word, 0);
   if (NULL == list)
      printf("Nothing found for '%s'\n", word);
   else
      {
      printf("list size is %d\n", list_size(list));
      struct list_iterator * it = list_begin(list);
      struct list_iterator * next = it;
      while (NULL != next)
	 {
	 printf("%s\n", list_value(it));
	 next = list_next(it);
	 }
      ctrie_free_subkey_list(list);
      }

   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "a");
   printf("Looking for first 5 completions of '%s':\n", word);
   list = ctrie_get_subkeys(ctrie, word, 5);
   if (NULL == list)
      printf("Nothing found for '%s'\n", word);
   else
      {
      printf("list size is %d\n", list_size(list));
      struct list_iterator * it = list_begin(list);
      struct list_iterator * next = it;
      while (NULL != next)
	 {
	 printf("%s\n", list_value(it));
	 next = list_next(it);
	 }
      ctrie_free_subkey_list(list);
      }

   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "anybody");
   printf("remove '%s'\n", word);
   ctrie_remove(ctrie, word);

   strcpy(word, "ann");
   printf("remove '%s'\n", word);
   ctrie_remove(ctrie, word);

   strcpy(word, "help");
   printf("remove '%s'\n", word);
   ctrie_remove(ctrie, word);

   strcpy(word, "a");
   printf("remove '%s'\n", word);
   ctrie_remove(ctrie, word);

   ctrie_dump(ctrie);
   printf("ctrie is storing %zd words in %zd keys\n", ctrie_count(ctrie), ctrie_size(ctrie));

   ////////////////////////////////////////////
   printf("===================================================\n");
   strcpy(word, "anybody");
   printf("re-inserting '%s'\n", word);
   ctrie_insert(ctrie, word, "fake definition");
   ctrie_dump(ctrie);
   printf("ctrie is storing %zd words in %zd keys\n", ctrie_count(ctrie), ctrie_size(ctrie));

   exit(EXIT_SUCCESS);
   }
